#!/usr/bin/perl
#######################################################
# Checksums                    Mike Machado           #
#  1.0                         mike@innercite.com     #
#                              2000-01-07             #
#                                                     #
#    Checks a file of MD5 sums against                #
#    the system to detect differences                 #
#                                                     #
# Copywrite under the terms of the GNU public license #
#                                                     #
#######################################################

use CGI;
use strict;

#### Options ####
my $requirepass = 1;		# either 1 or 0 (only used when a cgi)
my $password = 'test';		# only used in cgi mode with requirepass = 1
my $md5sum = '/usr/bin/md5sum';	# path to md5sum program


#### Globals ####
my $mode = 'cli';
my $verbose = 0;
my $query = '';
my $cgi = '';
my $body = '<BODY BGCOLOR="FFFFFF">';

$mode = 'cgi' if $ENV{'HTTP_HOST'};

if ($mode eq 'cgi') {
	$query = new CGI;
	$cgi = $query->url();
}

print "Content-type: text/html\n\n" if $mode eq 'cgi';


if ($mode eq 'cli') {
	usage() if $ARGV[0] eq '';
	$verbose = 1 if $ARGV[0] eq '-v';
	usage() if $verbose && $ARGV[1] eq '';
	my $file = $ARGV[0];
	$file = $ARGV[1] if $verbose;

	readsumsfile($file);

} elsif ($mode eq 'cgi') {

	if ($query->param('check_sums') ne '') {

		print "<HTML><HEAD><TITLE>MD5 Sum Checker</TITLE></HEAD>\n";
		print "$body<CENTER><H2>MD5 Sum Checker</H2></CENTER>\n";
		error('password') if $query->param('password') ne $password && $requirepass;
		$verbose = 1 if $query->param('verbose');
		print "Uploaded file: ".$query->param('sumsfile')."<BR>\n" if $verbose;
		my $mismatch = 0;
		no strict;
		my $uploaded = $query->param('sumsfile');
		while (<$uploaded>) {
			chomp();
			next if $_ eq '';
			print "\n" if $verbose;
			print "<BR>\n" if $mode eq 'cgi' && $verbose;
	
			my ($sum, $checkfile) = split(/\s+/);
			print "  Checking $checkfile for sum of $sum\n" if $verbose;
			print "<BR>\n" if $mode eq 'cgi' && $verbose;
			if (!checksum($sum, $checkfile)) {
				$mismatch++;
				print "<BLINK><FONT COLOR='red'>WARNING</FONT></BLINK> Sums for '$checkfile' do not match!<BR>\n" if $mode eq 'cgi';
			}
		}
		use strict;

		print "<BR><BR>Total different checksums: $mismatch<BR>\n";

	} else {
		print "<HTML><HEAD><TITLE>MD5 Sum Checker</TITLE></HEAD>\n";
		print "$body<CENTER><H2>MD5 Sum Checker</H2></CENTER>\n";
		print "Choose file of sums and click 'Upload'<BR><BR>\n";
		print "Format is: <I>md5checksum /path/to/file/being/checked</I><BR>\n";
		print "One entry per line, as many lines as you want<BR>\n";
		print "<TABLE BORDER=0>\n";
		print "<FORM ACTION=\"$cgi\" METHOD=\"POST\" ENCTYPE=\"multipart/form-data\">\n";
		print "<TR><TD>Password:</TD><TD><INPUT TYPE=\"PASSWORD\" NAME=\"password\"></TD></TR>\n" if $requirepass;
		print "<TR><TD>File:</TD><TD><INPUT TYPE=\"FILE\" NAME=\"sumsfile\"></TD></TR>\n";
		print "<TR><TD><INPUT TYPE=\"SUBMIT\" VALUE=\"Upload\" NAME=\"check_sums\"></TD><TD><INPUT TYPE=\"CHECKBOX\" NAME=\"verbose\"> Verbose</TD></TR>\n";
		print "</FORM></TABLE></BODY></HTML>\n";
	}

} else {
	print "Error: Unknown access method\n";
	exit;
}

sub usage {
	if ($mode eq 'cli') {
		print "Usage: checkmd5.pl [-v] file\n";
		exit;
	} elsif ($mode eq 'cgi') {
		print "<CENTER><H2>Error!</H2></CENTER>\n";
		print "<I>You must choose a file to be uploaded. Please click back and choose a file</I>\n";
	} else {
		print "Error: Unknown access method\n";
	}
}

sub readsumsfile {
	my $file = shift;

	print "  Opening sums file $file...\n" if $verbose;
	print "<BR>\n" if $mode eq 'cgi' && $verbose;
	open(SUMS, $file) || error('file', $file);
	my $mismatch = 0;
	while (<SUMS>) {
		chomp();
		next if $_ eq '';

		print "\n" if $verbose;
		print "<BR>\n" if $mode eq 'cgi' && $verbose;

		my ($sum, $checkfile) = split(/\s+/);
		print "  Checking $checkfile for sum of $sum\n" if $verbose;
		print "<BR>\n" if $mode eq 'cgi' && $verbose;
		if (!checksum($sum, $checkfile)) {
			$mismatch++;
			print "Error: Sums for '$checkfile' do not match!\n" if $mode eq 'cli';
			print "<H3><BLINK><FONT COLOR='red'>WARNING</FONT></BLINK> Sums for '$checkfile' do not match!</H3><BR>\n" if $mode eq 'cgi';
		}
	}
	close(SUMS);

	print "  \nTotal different checksums: $mismatch\n" if $verbose;
}

sub checksum {
	my $sum = shift;
	my $file = shift;
	
	if ($file =~ /^([-\w\/.]+)$/) {
		$file = $1;                     # $log now untainted
	} else {
		print "Error: Invalid file: $file\n";
		print "<BR>\n" if $mode eq 'cgi';
		return 2;
	}

	open(MD5SUM, "$md5sum $file 2>&1|") || error('md5sum');
	chomp(my $result = <MD5SUM>);
	print "  Got result '$result'\n" if $verbose;
	print "<BR>\n" if $verbose && $mode eq 'cgi';
	if ($result =~ /(.*)Permission denied(.*)/) {
		print "Permission denied while trying to md5sum '$file'\n";
		print "<BR>\n" if $mode eq 'cgi';
		return 2;
	}
	if ($result =~ /(.*)No such file or directory(.*)/) {
		print "File '$file' not found\n";
		print "<BR>\n" if $mode eq 'cgi';
		return 2;
	}
	if ($result =~ /(.*)Not a directory(.*)/) {
		print "File error on '$file'\n";
		print "<BR>\n" if $mode eq 'cgi';
		return 2;
	}
	my ($s, $f) = split(/\s+/, $result);
	if ($f ne $file) {
		print "Unknown error while trying to compair '$file'\n";
		print "<BR>\n" if $mode eq 'cgi';
		return 2;
	}
		
	print "  File $f currently has sum $s\n" if $verbose;
	print "<BR>\n" if $mode eq 'cgi' && $verbose;

	return 0 if $s ne $sum;
	return 1;
}

sub error {
	my $error = shift;

	if ($error eq 'file') {
		my $file = shift;
		print "Error opening file '$file' for reading\n" if $mode eq 'cli';
		print "<CENTER><H2>Error open file <I>$file</I> for reading</H2><CENTER>\n" if $mode eq 'cgi';
	} elsif ($error eq 'md5sum') {
		print "Error executing $md5sum\n" if $mode eq 'cli';
		print "<CENTER><H2>Error executing $md5sum</H2><CENTER>\n" if $mode eq 'cgi';
	} elsif ($error eq 'password') {
		print "<CENTER><H2>Error: Invalid password. Please go back and try again.</H2><CENTER>\n" if $mode eq 'cgi';
	}
	exit;
}
